#include "ProdCons.hpp"

using namespace std;

#ifndef __ERROR_MACRO_PRODCONS
#define __ERROR_MACRO_PRODCONS
void
__error_(const char *file, const int ligne, const char *msg, std::error_code errnum, int sortie);

/// Si errnum est != zero, cette fonction affiche le message et sort
#define exit_error(msg, errnum) __error_prodcons(__FILE__, __LINE__, (msg), errnum, 1)
/// Si errnum est != zero, cette fonction affiche le message sans sortir du programme
#define warning_error(msg, errnum) __error_prodcons(__FILE__, __LINE__, (msg), errnum, 0)

// ##################################
// gestion des erreurs
// ##################################
void
inline __error_prodcons(const char *file, const int ligne, const char *msg, std::error_code errnum, int sortie) {
  if (errnum) {
    std::cerr <<  "ERREUR fichier " << file << " ligne " <<  ligne << " : " << msg << " car " << errnum.message() << std::endl;
    if (sortie) {
      //      throw boost::system::system_error(boost::system::error_condition(errnum));
      throw std::system_error(errnum);
    }
  }
}

#endif


template <typename contenu>
ProdCons<contenu>::ProdCons(int taille) : taille(taille), data(),
				 mut(),
				 vide(),
				 plein()
{
  
}

template <typename contenu>
ProdCons<contenu>::~ProdCons() {
  cerr << "Destruction du ProdCons" << endl;
}

template <typename contenu>
contenu
ProdCons<contenu>::get() {
  std::unique_lock<std::mutex> unk(mut);

  while (data.size() == 0) {
    vide.wait(unk);
  }

  contenu result = data.front();
  data.pop_front();

  plein.notify_one();

  return result;
}
  

/**
 * @brief ajout d'un élément
 * @param c : l'élément à ajouter, il est copié par l'opérateur =
 */
template <typename T>
void
ProdCons<T>::put(const T &c) {
  std::unique_lock<std::mutex> unk(mut);

  while (data.size() == taille) {
    plein.wait(unk);
  }

  data.push_back(c);

  vide.notify_one();

}

