#pragma once
#include <iostream>
#include <string>
#include <list>
#include <thread>
#include <mutex>
#include <condition_variable>



template <typename contenu>
class ProdCons {
  /// attributs
  unsigned int taille;
  std::list<contenu> data;
  std::mutex mut;
  std::condition_variable vide;
  std::condition_variable plein;

public:
  /**
     @brief constructeur
     @param taille : la taille max de la liste
  */
  ProdCons(int taille=10);

  /**
     @brief destructeur
  */
  ~ProdCons();

  /**
   * @brief consomation d'un élément
   * @return s'il y a au moins un élément disponible, renvoie l'élement, sinon
   * elle bloque
   */
  contenu get();

  // /**
  //  * @brief consomation d'un élément
  //  * @return s'il y a au moins un élément disponible, renvoie l'élement, sinon
  //  * elle envoie une exception
  //  */
  // contenu try_get();  
  

  /**
   * @brief ajout d'un élément
   * @param c : l'élément à ajouter, il est copié par l'opérateur =
   */
  void put(const contenu &c);


  /**
   * @brief operateur de recopie pour le pasage d'argument supprimé pour viter les copie d mutex
   */
  ProdCons(const ProdCons& pc) = delete;


private:
  /**
   * @brief opérateur d'affectation je le met privé pour évité qu'il soit utilisé supprimé pour eviter les copies de mutex
   */
  ProdCons &operator=(const ProdCons &pc) = delete;
};


