#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#include "image.hpp"

unsigned int image_coord_wrapped(image_t const image,
        int const line,
        int const col)
{
  unsigned int wrap_line = std::abs((long) line%image.h);
  unsigned int wrap_col = std::abs((long) col%image.w);

    unsigned int coord = wrap_line * image.w + wrap_col;

    assert(coord < image.h*image.w);

    return coord;
}


image_t image_new(unsigned int const w,
        unsigned int const h)
{
    assert(w > 0);
    assert(h > 0);

    image_t result;
    result.w = w;
    result.h = h;
    result.data = (pixel_t*) calloc(w*h, sizeof(pixel_t));

    assert(result.data);

    return result;
}

void image_free(image_t const im)
{
    free(im.data);
}

image_t image_from_file(char const * const filename)
{
    unsigned int w,h,depth;
    FILE*fp = fopen(filename,"rb");
    assert(fp);

    assert(fscanf(fp,"P5\n%d %d\n%d\n",&w,&h,&depth) == 3);

    image_t im = image_new(w,h);
    unsigned int nb_reads = 0;

    if(depth == 65535)
    {
        unsigned short *data = (unsigned short*) malloc(sizeof(unsigned short) * w * h);
        while(fread(data + nb_reads, sizeof(*data), w*h - nb_reads,fp) != 0);
	unsigned int i;
        for(i=0; i < w*h; i++)
        {
            unsigned short tmp;
            unsigned char* tmp_offset = (unsigned char*) &tmp;
            unsigned char* val_offset = (unsigned char*) &(data[i]);

            tmp_offset[0] = val_offset[1];
            tmp_offset[1] = val_offset[0];

            im.data[i] = ((int) tmp) * 256 / 65536;
        }
        free(data);
    }
    else {
        while(fread(im.data + nb_reads, sizeof(*im.data), w*h - nb_reads,fp) != 0);
    }

    assert(!ferror(fp));
    fclose(fp);

    return im;
}

void image_to_file(image_t const im, char const * const filename)
{
    FILE*fp = fopen(filename,"wb");
    assert(fp);

    unsigned int nb_writes = 0;
    fprintf(fp,"P5\n%d %d\n255\n",im.w,im.h);

    while(nb_writes < im.w * im.h)
    {
        nb_writes += fwrite(im.data + nb_writes,sizeof(pixel_t),im.w*im.h - nb_writes,fp);
        assert(!ferror(fp));
    }

    fclose(fp);
}

// *************************************************
/*
 * 	Etant donne les coordonnees d'un point a+ib dans le plan
 * 	complexe, retourne la couleur correspondante estimant
 *	a quel distance de l'ensemble de mandelbrot le point est:
 *	En definisant la suite  ainsi (Z=x+iy):
 *		Z(0) = 0
 *		Z(n+1) = Z(n) * Z(n) - (a +ib)
 *  
 *	le nombre d'iterations que la suite met pour diverger
 *	correspond a une couleur dans la palette des couleurs
 *
 */
unsigned char xy2color(double a, double b, int prof) {
  double x, y, temp, x2, y2;
  int i;

  x = y = 0.;
  for( i=0; i<prof; i++) {
	/* garder la valeur précédente de x qui va etre ecrase */
	temp = x;
	/* nouvelles valeurs de x et y */
	x2 = x*x;
	y2 = y*y;
	x = x2 - y2 + a;
	y = 2*temp*y + b;
	if( x2 + y2 >= 4.0) break;
  }
  return i*255/prof;
}
